@@ -8,10 +8,10 @@ Dir.chdir(dir)
 require './tc_attr'
 require './tc_attr_decl'
 require './tc_attributes'
+require './tc_canonicalize'
 require './tc_document'
 require './tc_document_write'
 require './tc_dtd'
-require './tc_encoding'
 require './tc_error'
 require './tc_html_parser'
 require './tc_html_parser_context'
@@ -22,6 +22,7 @@ require './tc_node_cdata'
 require './tc_node_comment'
 require './tc_node_copy'
 require './tc_node_edit'
+require './tc_node_pi'
 require './tc_node_text'
 require './tc_node_write'
 require './tc_node_xlink'
@@ -38,6 +39,11 @@ require './tc_xpath_context'
 require './tc_xpath_expression'
 require './tc_xpointer'
 
+
+if defined?(Encoding)
+  require './tc_encoding'
+  require './tc_encoding_sax'
+end
 # Compatibility
 require './tc_properties'
 require './tc_deprecated_require'
\ No newline at end of file
